// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"context"
	"fmt"
	"os"

	"github.com/spf13/cobra"
)

var envelopeCmd = &cobra.Command{
	Use:   "envelope",
	Short: "Envelope management",
}

var envelopePurgeCmd = &cobra.Command{
	Use:   "purge",
	Short: "Remove envelopes that are not in an existing process",
	Run: func(cmd *cobra.Command, args []string) {
		c, err := GetClient()
		if err != nil {
			return
		}
		ctx, _ := contextCancelOnSignal(
			context.Background(), os.Interrupt,
		)
		var count uint32
		fmt.Print("\nPurge: ")
		err = c.EnvelopePurge(
			ctx, func(c uint32) {
				fmt.Printf("\rPurge: %d", c)
				count = c
			},
		)
		if err != nil {
			Logger.Errorf("%s", err)
			return
		}
		Logger.Noticef("Purged %d envelope(s)", count)
	},
}

func init() {
	RootCmd.AddCommand(envelopeCmd)
	envelopeCmd.AddCommand(envelopePurgeCmd)
}
