// Copyright 2017 Orus.io. All rights reserved.

package broker

import (
	"context"

	"bitbucket.org/orus-io/go-xbus/api"
)

type actorProcessServer struct {
	*Service
}

func (s actorProcessServer) Log(
	ctx context.Context, req api.ActorLogRequest,
) (err error) {
	actorSID := ctx.Value("nrpc-svc-actorID").(string)
	actorID, err := api.UUIDFromString(actorSID)
	if err != nil {
		return
	}

	if req.EnvelopeID != nil && !req.EnvelopeID.IsZero() {
		_, err = s.Opts.ProcessManager.Get(req.Context.ProcessID)
		if err != nil {
			return
		}
	}

	var (
		envID, processID api.UUID
		nodeID           string
	)
	if req.EnvelopeID != nil {
		envID = *req.EnvelopeID
	}
	if ctx := req.Context; ctx != nil {
		nodeID = ctx.NodeID
		processID = ctx.ProcessID
	}
	for _, msg := range req.Messages {
		s.Opts.LogStorage.Log(api.LogEntry{
			EnvelopeID: envID,
			ActorID:    actorID,
			NodeID:     nodeID,
			ProcessID:  processID,
			Message:    msg,
		})
	}
	return
}

func (s actorProcessServer) ProcessingEnd(
	ctx context.Context, req api.ActorProcessingEndRequest,
) (err error) {
	actorSID := ctx.Value("nrpc-svc-actorID").(string)
	actorID, err := api.UUIDFromString(actorSID)
	if err != nil {
		return
	}

	p, err := s.Opts.ProcessManager.Get(req.Context.ProcessID)
	if err != nil {
		return
	}

	for _, msg := range req.Messages {
		s.Opts.LogStorage.Log(api.LogEntry{
			EnvelopeID: req.EnvelopeID,
			ActorID:    actorID,
			NodeID:     req.Context.NodeID,
			ProcessID:  req.Context.ProcessID,
			Message:    msg,
		})
	}

	p.ActorProcessingEnd(req.EnvelopeID, actorID, req.Context.NodeID, req.Result)
	return
}
