// Copyright 2016 Orus.io. All rights reserved.

package control

import (
	"context"
	"fmt"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/go-xbus/api/control"
)

type actorServer struct {
	*Service
}

func (s actorServer) Create(ctx context.Context, req control.ActorRequest) (resp control.ActorReply, err error) {
	return
}

func (s actorServer) Update(ctx context.Context, req control.ActorRequest) (resp control.ActorReply, err error) {
	for _, actor := range req.ActorList {
		var sActor api.Actor
		if actor.Status == api.Actor_ENABLED {
			sActor, err = s.actorAccept(actor.ID)
		} else if actor.Status == api.Actor_REJECTED {
			sActor, err = s.actorReject(actor.ID)
		} else {
			err = api.ClientProtoError("Invalid actor update")
			return
		}
		actor = api.Actor{
			ID:     sActor.ID,
			Name:   sActor.Name,
			Kind:   sActor.Kind,
			Status: sActor.Status,
			Roles:  sActor.Roles,
		}

		if err != nil {
			break
		}
		resp.ActorList = append(resp.ActorList, actor)
	}
	return
}

// Delete deletes an Actor
func (s actorServer) Delete(ctx context.Context, req control.ActorRequest) (resp control.ActorReply, err error) {
	for _, actor := range req.ActorList {
		s.o.ActorStorage.Delete(actor.ID)
	}
	return
}

// Query lis Actors
func (s actorServer) Query(ctx context.Context, req control.ActorRequest) (resp control.ActorReply, err error) {
	resp.ActorList, err = s.actorQuery(req.GetActorList())
	return
}

func (s *Service) actorQuery(filters []api.Actor) (res []api.Actor, err error) {
	if len(filters) == 0 {
		return s.o.ActorStorage.List(), nil
	}
	for _, filter := range filters {
		switch {
		case !filter.ID.IsZero():
			if actor, exis := s.o.ActorStorage.Get(filter.ID); exis {
				res = append(res, actor)
			}
		case filter.Name != "":
			if actor, exis := s.o.ActorStorage.GetByName(filter.Name); exis {
				res = append(res, actor)
			}
		case !filter.AccountID.IsZero():
			res = append(res, s.o.ActorStorage.ListAccount(filter.AccountID)...)
		default:
			panic("invalid filter")
		}
	}
	return
}

func (s *Service) actorAccept(id api.UUID) (api.Actor, error) {
	a, exis := s.o.ActorStorage.Get(id)
	if !exis {
		return api.Actor{}, fmt.Errorf("No such actor: %s", id)
	}
	if a.Status != api.Actor_PENDING {
		return api.Actor{}, fmt.Errorf("Invalid initial status '%s' for 'Accept'", a.Status)
	}
	a.Status = api.Actor_ENABLED
	return s.o.ActorStorage.Update(a)
}

func (s *Service) actorReject(id api.UUID) (api.Actor, error) {
	a, exis := s.o.ActorStorage.Get(id)
	if !exis {
		return api.Actor{}, fmt.Errorf("No such actor: %s", id)
	}
	if a.Status != api.Actor_PENDING {
		return api.Actor{}, fmt.Errorf("Invalid initial status '%s' for 'Reject'", a.Status)
	}
	a.Status = api.Actor_REJECTED
	return s.o.ActorStorage.Update(a)
}
