// Copyright 2016 Orus.io. All rights reserved.

package sentinel

import (
	"context"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/services"
	"bitbucket.org/orus-io/xbus/storage"
)

const (
	// ServiceName name of the service
	ServiceName = "sentinel"

	// QueueName queue name for queuesubscribes
	QueueName = "xbus.services." + ServiceName
)

// Options is used to create a 'processstate' service
type Options struct {
	services.Options

	ActorStorage storage.ActorStorage
}

// NewService creates a Service
func (o Options) NewService() *Service {
	return &Service{
		BaseService: services.NewBaseService(ServiceName, o.Options),
		o:           o,
	}
}

// Service is in charge of emitting process status events to various subjects
type Service struct {
	services.BaseService

	o Options
}

// Startup starts the service
func (s *Service) Startup() error {
	sentinelHandler := api.NewSentinelHandler(
		context.Background(), s.Conn, sentinelServer{s},
	)
	return s.SubscribeAll(
		s.HandlerQueueSubscribe(sentinelHandler, QueueName),
	)
}

type sentinelServer struct {
	*Service
}

func (s sentinelServer) ActorReady(ctx context.Context, actorID string) {
	actorUUID, err := api.UUIDFromString(actorID)
	if err != nil {
		s.Log.Errorf("Received invalid actorID: %s (err: %s)", actorID, err)
	}
	// signal the actor was just seen
	if err := s.o.ActorStorage.JustSeen(actorUUID); err != nil {
		s.Log.Errorf("Error calling JustSeen: %s", err)
	}
}
