// Copyright 2016 Orus.io. All rights reserved.

package model

import (
	"time"

	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
)

//go:generate yago --fmt

// Actor is any client of the bus (emitter, consumer, worker)
//yago:
type Actor struct {
	Base
	Name      string           `yago:"unique_index=name"`
	AccountID uuid.NullUUID    `yago:"fk=Account ONDELETE SET NULL"`
	Kind      api.Actor_Kind   `yago:"textmarshaled"`
	Role      string           `yago:"."`
	Status    api.Actor_Status `yago:"textmarshaled"`

	Visible  bool      `yago:"."`
	LastSeen time.Time `yago:"."`
}

// ToAPI convert the actor to a api.Actor
func (a Actor) ToAPI() api.Actor {
	actor := api.Actor{
		ID:     api.UUID(a.ID),
		Name:   a.Name,
		Kind:   a.Kind,
		Roles:  []string{a.Role},
		Status: a.Status,
	}
	return actor
}

// AccountFromAPIActor provides a model.Account from an api.Actor.
// It is assumed the account and the actor have the same name.
// TODO To be reworked once we have an api.Account.
func AccountFromAPIActor(actor api.Actor) Account {
	return Account{
		Base: Base{ID: uuid.UUID(actor.ID)},
		Name: actor.Name,
	}
}

// ActorFromAPI convert a api.Actor to model.Actor
func ActorFromAPI(actor api.Actor) Actor {
	return Actor{
		Base:   Base{ID: uuid.UUID(actor.ID)},
		Kind:   actor.Kind,
		Name:   actor.Name,
		Role:   actor.Roles[0],
		Status: actor.Status,
	}
}

// ActorListToAPI converts a []Actor to []api.Actor
func ActorListToAPI(actors []Actor) (res []api.Actor) {
	for _, a := range actors {
		res = append(res, a.ToAPI())
	}
	return
}
