package storage

import (
	"testing"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/storage"
)

var simpleEnvelope = api.Envelope{
	ID: api.MustUUIDFromString("b74415ee-83c1-11e6-8f04-133b336287df"),
	Events: []api.Event{
		api.Event{
			ID:        api.MustUUIDFromString("c2e373fe-83c1-11e6-a098-c3d72ea43882"),
			Index:     1,
			Type:      "xbus.test",
			Items:     []api.Item{[]byte("Data")},
			ItemCount: 1,
			Checksum:  1859455373,
		},
	},
}

// EnvelopeStorageBench is a benchmark suite for a EnvelopeStorage implementation
func EnvelopeStorageBench(b *testing.B, factory EnvelopeStorageFactory) {
	b.Run("InitClearStorage", func(b *testing.B) {
		for i := 0; i != b.N; i++ {
			_, clean := factory(b)
			clean()
		}
	})
	b.Run("StoreAndReadOneSimpleEnvelope", func(b *testing.B) {
		for i := 0; i != b.N; i++ {
			es, clean := factory(b)
			es.StoreEnvelope(simpleEnvelope)
			es.ReadEnvelope(simpleEnvelope.ID, storage.EnvelopeStartPos(), 1024*1024)
			clean()
		}
	})
	b.Run("StoreAndReadManySimpleEnvelope", func(b *testing.B) {
		var err error
		es, clean := factory(b)
		defer clean()
		for i := 0; i != b.N; i++ {
			simpleEnvelope.ID, err = api.UUIDV4()
			if err != nil {
				panic(err)
			}
			es.StoreEnvelope(simpleEnvelope)
			es.ReadEnvelope(simpleEnvelope.ID, storage.EnvelopeStartPos(), 1024*1024)
		}
	})
}
