// This code was autogenerated from xbus/control.proto, do not edit.
package control

import (
	"context"
	"log"
	"time"

	"github.com/golang/protobuf/proto"
	nats "github.com/nats-io/go-nats"
	bitbucket_org_orus_io_go_xbus_api "bitbucket.org/orus-io/go-xbus/api"
	github_com_nats_rpc_gogo_nrpc "github.com/nats-rpc/gogo-nrpc"
	"github.com/prometheus/client_golang/prometheus"
	"github.com/nats-rpc/gogo-nrpc"
)

// AccountServer is the interface that providers of the service
// Account should implement.
type AccountServer interface {
	List(ctx context.Context, req AccountListRequest) (resp AccountListReply, err error)
	Update(ctx context.Context, req AccountUpdateRequest) (resp bitbucket_org_orus_io_go_xbus_api.Account, err error)
	Delete(ctx context.Context, req AccountIDRequest) (err error)
	RenewAPIKey(ctx context.Context, req AccountIDRequest) (resp bitbucket_org_orus_io_go_xbus_api.Account, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForAccount = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Account",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForAccount = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Account",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForAccount = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Account",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForAccount = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Account",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// AccountHandler provides a NATS subscription handler that can serve a
// subscription using a given AccountServer implementation.
type AccountHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server AccountServer
}

func NewAccountHandler(ctx context.Context, nc nrpc.NatsConn, s AccountServer) *AccountHandler {
	return &AccountHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *AccountHandler) Subject() string {
	return "xbus.control.account.>"
}

func (h *AccountHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus.control", 0, "account", 0, msg.Subject)
	if err != nil {
		log.Printf("AccountHanlder: Account subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "list":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("ListHanlder: List subject parsing failed: %v", err)
			break
		}
		var req AccountListRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("ListHandler: List request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForAccount.WithLabelValues(
				"List", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.List(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("ListHandler: List handler failed: %s", replyError.Error())
				serverRequestsForAccount.WithLabelValues(
					"List", encoding, "handler_fail").Inc()
			}
		}
	case "update":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("UpdateHanlder: Update subject parsing failed: %v", err)
			break
		}
		var req AccountUpdateRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("UpdateHandler: Update request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForAccount.WithLabelValues(
				"Update", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Update(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("UpdateHandler: Update handler failed: %s", replyError.Error())
				serverRequestsForAccount.WithLabelValues(
					"Update", encoding, "handler_fail").Inc()
			}
		}
	case "delete":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("DeleteHanlder: Delete subject parsing failed: %v", err)
			break
		}
		var req AccountIDRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("DeleteHandler: Delete request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForAccount.WithLabelValues(
				"Delete", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					var innerResp nrpc.Void
					err := h.server.Delete(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("DeleteHandler: Delete handler failed: %s", replyError.Error())
				serverRequestsForAccount.WithLabelValues(
					"Delete", encoding, "handler_fail").Inc()
			}
		}
	case "renew_apikey":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("RenewAPIKeyHanlder: RenewAPIKey subject parsing failed: %v", err)
			break
		}
		var req AccountIDRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("RenewAPIKeyHandler: RenewAPIKey request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForAccount.WithLabelValues(
				"RenewAPIKey", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.RenewAPIKey(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("RenewAPIKeyHandler: RenewAPIKey handler failed: %s", replyError.Error())
				serverRequestsForAccount.WithLabelValues(
					"RenewAPIKey", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("AccountHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForAccount.WithLabelValues(
			"Account", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForAccount.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForAccount.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForAccount.WithLabelValues(name).Observe(elapsed)
}

type AccountClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewAccountClient(nc nrpc.NatsConn) *AccountClient {
	return &AccountClient{
		nc:      nc,
		PkgSubject: "xbus.control",
		Subject: "account",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *AccountClient) List(req AccountListRequest) (resp AccountListReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "list";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForAccount.WithLabelValues(
			"List", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForAccount.WithLabelValues("List").Observe(elapsed)
	clientCallsForAccount.WithLabelValues(
		"List", c.Encoding, "success").Inc()

	return
}

func (c *AccountClient) Update(req AccountUpdateRequest) (resp bitbucket_org_orus_io_go_xbus_api.Account, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "update";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForAccount.WithLabelValues(
			"Update", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForAccount.WithLabelValues("Update").Observe(elapsed)
	clientCallsForAccount.WithLabelValues(
		"Update", c.Encoding, "success").Inc()

	return
}

func (c *AccountClient) Delete(req AccountIDRequest) (err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "delete";

	// call
	var resp github_com_nats_rpc_gogo_nrpc.Void
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForAccount.WithLabelValues(
			"Delete", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForAccount.WithLabelValues("Delete").Observe(elapsed)
	clientCallsForAccount.WithLabelValues(
		"Delete", c.Encoding, "success").Inc()

	return
}

func (c *AccountClient) RenewAPIKey(req AccountIDRequest) (resp bitbucket_org_orus_io_go_xbus_api.Account, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "renew_apikey";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForAccount.WithLabelValues(
			"RenewAPIKey", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForAccount.WithLabelValues("RenewAPIKey").Observe(elapsed)
	clientCallsForAccount.WithLabelValues(
		"RenewAPIKey", c.Encoding, "success").Inc()

	return
}

// ActorServer is the interface that providers of the service
// Actor should implement.
type ActorServer interface {
	Create(ctx context.Context, req ActorRequest) (resp ActorReply, err error)
	Update(ctx context.Context, req ActorRequest) (resp ActorReply, err error)
	Delete(ctx context.Context, req ActorRequest) (resp ActorReply, err error)
	Query(ctx context.Context, req ActorRequest) (resp ActorReply, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForActor = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Actor",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForActor = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Actor",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForActor = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Actor",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForActor = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Actor",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// ActorHandler provides a NATS subscription handler that can serve a
// subscription using a given ActorServer implementation.
type ActorHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server ActorServer
}

func NewActorHandler(ctx context.Context, nc nrpc.NatsConn, s ActorServer) *ActorHandler {
	return &ActorHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *ActorHandler) Subject() string {
	return "xbus.control.actor.>"
}

func (h *ActorHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus.control", 0, "actor", 0, msg.Subject)
	if err != nil {
		log.Printf("ActorHanlder: Actor subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "create":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("CreateHanlder: Create subject parsing failed: %v", err)
			break
		}
		var req ActorRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("CreateHandler: Create request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForActor.WithLabelValues(
				"Create", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Create(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("CreateHandler: Create handler failed: %s", replyError.Error())
				serverRequestsForActor.WithLabelValues(
					"Create", encoding, "handler_fail").Inc()
			}
		}
	case "update":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("UpdateHanlder: Update subject parsing failed: %v", err)
			break
		}
		var req ActorRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("UpdateHandler: Update request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForActor.WithLabelValues(
				"Update", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Update(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("UpdateHandler: Update handler failed: %s", replyError.Error())
				serverRequestsForActor.WithLabelValues(
					"Update", encoding, "handler_fail").Inc()
			}
		}
	case "delete":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("DeleteHanlder: Delete subject parsing failed: %v", err)
			break
		}
		var req ActorRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("DeleteHandler: Delete request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForActor.WithLabelValues(
				"Delete", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Delete(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("DeleteHandler: Delete handler failed: %s", replyError.Error())
				serverRequestsForActor.WithLabelValues(
					"Delete", encoding, "handler_fail").Inc()
			}
		}
	case "query":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("QueryHanlder: Query subject parsing failed: %v", err)
			break
		}
		var req ActorRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("QueryHandler: Query request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForActor.WithLabelValues(
				"Query", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Query(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("QueryHandler: Query handler failed: %s", replyError.Error())
				serverRequestsForActor.WithLabelValues(
					"Query", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("ActorHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForActor.WithLabelValues(
			"Actor", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForActor.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForActor.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForActor.WithLabelValues(name).Observe(elapsed)
}

type ActorClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewActorClient(nc nrpc.NatsConn) *ActorClient {
	return &ActorClient{
		nc:      nc,
		PkgSubject: "xbus.control",
		Subject: "actor",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *ActorClient) Create(req ActorRequest) (resp ActorReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "create";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForActor.WithLabelValues(
			"Create", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForActor.WithLabelValues("Create").Observe(elapsed)
	clientCallsForActor.WithLabelValues(
		"Create", c.Encoding, "success").Inc()

	return
}

func (c *ActorClient) Update(req ActorRequest) (resp ActorReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "update";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForActor.WithLabelValues(
			"Update", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForActor.WithLabelValues("Update").Observe(elapsed)
	clientCallsForActor.WithLabelValues(
		"Update", c.Encoding, "success").Inc()

	return
}

func (c *ActorClient) Delete(req ActorRequest) (resp ActorReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "delete";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForActor.WithLabelValues(
			"Delete", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForActor.WithLabelValues("Delete").Observe(elapsed)
	clientCallsForActor.WithLabelValues(
		"Delete", c.Encoding, "success").Inc()

	return
}

func (c *ActorClient) Query(req ActorRequest) (resp ActorReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "query";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForActor.WithLabelValues(
			"Query", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForActor.WithLabelValues("Query").Observe(elapsed)
	clientCallsForActor.WithLabelValues(
		"Query", c.Encoding, "success").Inc()

	return
}

// EnvelopeServer is the interface that providers of the service
// Envelope should implement.
type EnvelopeServer interface {
	Purge(ctx context.Context, pushRep func(TaskProgress)) (err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForEnvelope = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Envelope",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForEnvelope = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Envelope",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForEnvelope = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Envelope",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForEnvelope = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Envelope",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// EnvelopeHandler provides a NATS subscription handler that can serve a
// subscription using a given EnvelopeServer implementation.
type EnvelopeHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server EnvelopeServer
}

func NewEnvelopeHandler(ctx context.Context, nc nrpc.NatsConn, s EnvelopeServer) *EnvelopeHandler {
	return &EnvelopeHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *EnvelopeHandler) Subject() string {
	return "xbus.control.envelope.>"
}

func (h *EnvelopeHandler) PurgeHandler(ctx context.Context, tail []string, msg *nats.Msg) {
	_, encoding, err := nrpc.ParseSubjectTail(0, tail)
	if err != nil {
		log.Printf("Envelope: Purge subject parsing failed:")
	}

	ctx, cancel := context.WithCancel(ctx)

	keepStreamAlive := nrpc.NewKeepStreamAlive(h.nc, msg.Reply, encoding, cancel)

	var msgCount uint32

	_, nrpcErr := nrpc.CaptureErrors(func() (proto.Message, error) {
		err := h.server.Purge(ctx, func(rep TaskProgress){
				if err = nrpc.Publish(&rep, nil, h.nc, msg.Reply, encoding); err != nil {
					log.Printf("nrpc: error publishing response")
					cancel()
					return
				}
				msgCount++
			})
		return nil, err
	})
	keepStreamAlive.Stop()

	if nrpcErr != nil {
		nrpc.Publish(nil, nrpcErr, h.nc, msg.Reply, encoding)
	} else {
		nrpc.Publish(
			nil, &nrpc.Error{Type: nrpc.Error_EOS, MsgCount: msgCount},
			h.nc, msg.Reply, encoding)
	}
}

func (h *EnvelopeHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus.control", 0, "envelope", 0, msg.Subject)
	if err != nil {
		log.Printf("EnvelopeHanlder: Envelope subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "purge":
		h.PurgeHandler(ctx, tail, msg)
		return
	default:
		log.Printf("EnvelopeHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForEnvelope.WithLabelValues(
			"Envelope", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForEnvelope.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForEnvelope.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForEnvelope.WithLabelValues(name).Observe(elapsed)
}

type EnvelopeClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewEnvelopeClient(nc nrpc.NatsConn) *EnvelopeClient {
	return &EnvelopeClient{
		nc:      nc,
		PkgSubject: "xbus.control",
		Subject: "envelope",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *EnvelopeClient) Purge(
	ctx context.Context,
	cb func (context.Context, TaskProgress),
) error {
	start := time.Now()
	subject := c.PkgSubject + "." + c.Subject + "." + "purge";

	sub, err := nrpc.StreamCall(ctx, c.nc, subject, &nrpc.Void{}, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForEnvelope.WithLabelValues(
			"Purge", c.Encoding, "error").Inc()
		return err
	}

	var res TaskProgress
	for {
		err = sub.Next(&res)
		if err != nil {
			break
		}
		cb(ctx, res)
	}
	if err == nrpc.ErrEOS {
		err = nil
	}
	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForEnvelope.WithLabelValues("Purge").Observe(elapsed)
	clientCallsForEnvelope.WithLabelValues(
		"Purge", c.Encoding, "success").Inc()
	return err
}

// GraphServer is the interface that providers of the service
// Graph should implement.
type GraphServer interface {
	Query(ctx context.Context, req GraphQueryRequest) (resp GraphQueryReply, err error)
	Save(ctx context.Context, req GraphSaveRequest) (resp GraphSaveReply, err error)
	SetStatus(ctx context.Context, req bitbucket_org_orus_io_go_xbus_api.GraphInfo) (resp GraphSetStatusReply, err error)
	Get(ctx context.Context, req bitbucket_org_orus_io_go_xbus_api.GraphInfo) (resp bitbucket_org_orus_io_go_xbus_api.StringReply, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForGraph = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Graph",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForGraph = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Graph",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForGraph = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Graph",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForGraph = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Graph",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// GraphHandler provides a NATS subscription handler that can serve a
// subscription using a given GraphServer implementation.
type GraphHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server GraphServer
}

func NewGraphHandler(ctx context.Context, nc nrpc.NatsConn, s GraphServer) *GraphHandler {
	return &GraphHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *GraphHandler) Subject() string {
	return "xbus.control.graph.>"
}

func (h *GraphHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus.control", 0, "graph", 0, msg.Subject)
	if err != nil {
		log.Printf("GraphHanlder: Graph subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "query":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("QueryHanlder: Query subject parsing failed: %v", err)
			break
		}
		var req GraphQueryRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("QueryHandler: Query request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForGraph.WithLabelValues(
				"Query", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Query(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("QueryHandler: Query handler failed: %s", replyError.Error())
				serverRequestsForGraph.WithLabelValues(
					"Query", encoding, "handler_fail").Inc()
			}
		}
	case "save":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("SaveHanlder: Save subject parsing failed: %v", err)
			break
		}
		var req GraphSaveRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("SaveHandler: Save request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForGraph.WithLabelValues(
				"Save", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Save(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("SaveHandler: Save handler failed: %s", replyError.Error())
				serverRequestsForGraph.WithLabelValues(
					"Save", encoding, "handler_fail").Inc()
			}
		}
	case "setstatus":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("SetStatusHanlder: SetStatus subject parsing failed: %v", err)
			break
		}
		var req bitbucket_org_orus_io_go_xbus_api.GraphInfo
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("SetStatusHandler: SetStatus request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForGraph.WithLabelValues(
				"SetStatus", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.SetStatus(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("SetStatusHandler: SetStatus handler failed: %s", replyError.Error())
				serverRequestsForGraph.WithLabelValues(
					"SetStatus", encoding, "handler_fail").Inc()
			}
		}
	case "get":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("GetHanlder: Get subject parsing failed: %v", err)
			break
		}
		var req bitbucket_org_orus_io_go_xbus_api.GraphInfo
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("GetHandler: Get request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForGraph.WithLabelValues(
				"Get", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Get(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("GetHandler: Get handler failed: %s", replyError.Error())
				serverRequestsForGraph.WithLabelValues(
					"Get", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("GraphHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForGraph.WithLabelValues(
			"Graph", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForGraph.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForGraph.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForGraph.WithLabelValues(name).Observe(elapsed)
}

type GraphClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewGraphClient(nc nrpc.NatsConn) *GraphClient {
	return &GraphClient{
		nc:      nc,
		PkgSubject: "xbus.control",
		Subject: "graph",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *GraphClient) Query(req GraphQueryRequest) (resp GraphQueryReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "query";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForGraph.WithLabelValues(
			"Query", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForGraph.WithLabelValues("Query").Observe(elapsed)
	clientCallsForGraph.WithLabelValues(
		"Query", c.Encoding, "success").Inc()

	return
}

func (c *GraphClient) Save(req GraphSaveRequest) (resp GraphSaveReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "save";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForGraph.WithLabelValues(
			"Save", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForGraph.WithLabelValues("Save").Observe(elapsed)
	clientCallsForGraph.WithLabelValues(
		"Save", c.Encoding, "success").Inc()

	return
}

func (c *GraphClient) SetStatus(req bitbucket_org_orus_io_go_xbus_api.GraphInfo) (resp GraphSetStatusReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "setstatus";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForGraph.WithLabelValues(
			"SetStatus", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForGraph.WithLabelValues("SetStatus").Observe(elapsed)
	clientCallsForGraph.WithLabelValues(
		"SetStatus", c.Encoding, "success").Inc()

	return
}

func (c *GraphClient) Get(req bitbucket_org_orus_io_go_xbus_api.GraphInfo) (resp bitbucket_org_orus_io_go_xbus_api.StringReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "get";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForGraph.WithLabelValues(
			"Get", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForGraph.WithLabelValues("Get").Observe(elapsed)
	clientCallsForGraph.WithLabelValues(
		"Get", c.Encoding, "success").Inc()

	return
}

// LogsServer is the interface that providers of the service
// Logs should implement.
type LogsServer interface {
	Purge(ctx context.Context, req LogsPurgeRequest, pushRep func(LogsPurgeReply)) (err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForLogs = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Logs",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForLogs = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Logs",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForLogs = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Logs",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForLogs = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Logs",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// LogsHandler provides a NATS subscription handler that can serve a
// subscription using a given LogsServer implementation.
type LogsHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server LogsServer
}

func NewLogsHandler(ctx context.Context, nc nrpc.NatsConn, s LogsServer) *LogsHandler {
	return &LogsHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *LogsHandler) Subject() string {
	return "xbus.control.logs.>"
}

func (h *LogsHandler) PurgeHandler(ctx context.Context, tail []string, msg *nats.Msg) {
	_, encoding, err := nrpc.ParseSubjectTail(0, tail)
	if err != nil {
		log.Printf("Logs: Purge subject parsing failed:")
	}
	var req LogsPurgeRequest
	if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
		// Handle error
		return
	}

	ctx, cancel := context.WithCancel(ctx)

	keepStreamAlive := nrpc.NewKeepStreamAlive(h.nc, msg.Reply, encoding, cancel)

	var msgCount uint32

	_, nrpcErr := nrpc.CaptureErrors(func() (proto.Message, error) {
		err := h.server.Purge(ctx, req, func(rep LogsPurgeReply){
				if err = nrpc.Publish(&rep, nil, h.nc, msg.Reply, encoding); err != nil {
					log.Printf("nrpc: error publishing response")
					cancel()
					return
				}
				msgCount++
			})
		return nil, err
	})
	keepStreamAlive.Stop()

	if nrpcErr != nil {
		nrpc.Publish(nil, nrpcErr, h.nc, msg.Reply, encoding)
	} else {
		nrpc.Publish(
			nil, &nrpc.Error{Type: nrpc.Error_EOS, MsgCount: msgCount},
			h.nc, msg.Reply, encoding)
	}
}

func (h *LogsHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus.control", 0, "logs", 0, msg.Subject)
	if err != nil {
		log.Printf("LogsHanlder: Logs subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "purge":
		h.PurgeHandler(ctx, tail, msg)
		return
	default:
		log.Printf("LogsHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForLogs.WithLabelValues(
			"Logs", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForLogs.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForLogs.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForLogs.WithLabelValues(name).Observe(elapsed)
}

type LogsClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewLogsClient(nc nrpc.NatsConn) *LogsClient {
	return &LogsClient{
		nc:      nc,
		PkgSubject: "xbus.control",
		Subject: "logs",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *LogsClient) Purge(
	ctx context.Context,
	req LogsPurgeRequest,
	cb func (context.Context, LogsPurgeReply),
) error {
	start := time.Now()
	subject := c.PkgSubject + "." + c.Subject + "." + "purge";

	sub, err := nrpc.StreamCall(ctx, c.nc, subject, &req, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForLogs.WithLabelValues(
			"Purge", c.Encoding, "error").Inc()
		return err
	}

	var res LogsPurgeReply
	for {
		err = sub.Next(&res)
		if err != nil {
			break
		}
		cb(ctx, res)
	}
	if err == nrpc.ErrEOS {
		err = nil
	}
	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForLogs.WithLabelValues("Purge").Observe(elapsed)
	clientCallsForLogs.WithLabelValues(
		"Purge", c.Encoding, "success").Inc()
	return err
}

// PMProcessServer is the interface that providers of the service
// PMProcess should implement.
type PMProcessServer interface {
	Query(ctx context.Context, req PMProcessQueryRequest) (resp PMProcessQueryReply, err error)
	SetStatus(ctx context.Context, req PMProcessSetStatusRequest) (err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForPMProcess = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "PMProcess",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForPMProcess = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "PMProcess",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForPMProcess = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "PMProcess",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForPMProcess = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "PMProcess",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// PMProcessHandler provides a NATS subscription handler that can serve a
// subscription using a given PMProcessServer implementation.
type PMProcessHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server PMProcessServer
}

func NewPMProcessHandler(ctx context.Context, nc nrpc.NatsConn, s PMProcessServer) *PMProcessHandler {
	return &PMProcessHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *PMProcessHandler) Subject() string {
	return "xbus.control.pm.>"
}

func (h *PMProcessHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus.control", 0, "pm", 0, msg.Subject)
	if err != nil {
		log.Printf("PMProcessHanlder: PMProcess subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "query":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("QueryHanlder: Query subject parsing failed: %v", err)
			break
		}
		var req PMProcessQueryRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("QueryHandler: Query request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForPMProcess.WithLabelValues(
				"Query", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Query(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("QueryHandler: Query handler failed: %s", replyError.Error())
				serverRequestsForPMProcess.WithLabelValues(
					"Query", encoding, "handler_fail").Inc()
			}
		}
	case "setstatus":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("SetStatusHanlder: SetStatus subject parsing failed: %v", err)
			break
		}
		var req PMProcessSetStatusRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("SetStatusHandler: SetStatus request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForPMProcess.WithLabelValues(
				"SetStatus", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					var innerResp nrpc.Void
					err := h.server.SetStatus(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("SetStatusHandler: SetStatus handler failed: %s", replyError.Error())
				serverRequestsForPMProcess.WithLabelValues(
					"SetStatus", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("PMProcessHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForPMProcess.WithLabelValues(
			"PMProcess", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForPMProcess.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForPMProcess.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForPMProcess.WithLabelValues(name).Observe(elapsed)
}

type PMProcessClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewPMProcessClient(nc nrpc.NatsConn) *PMProcessClient {
	return &PMProcessClient{
		nc:      nc,
		PkgSubject: "xbus.control",
		Subject: "pm",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *PMProcessClient) Query(req PMProcessQueryRequest) (resp PMProcessQueryReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "query";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForPMProcess.WithLabelValues(
			"Query", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForPMProcess.WithLabelValues("Query").Observe(elapsed)
	clientCallsForPMProcess.WithLabelValues(
		"Query", c.Encoding, "success").Inc()

	return
}

func (c *PMProcessClient) SetStatus(req PMProcessSetStatusRequest) (err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "setstatus";

	// call
	var resp github_com_nats_rpc_gogo_nrpc.Void
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForPMProcess.WithLabelValues(
			"SetStatus", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForPMProcess.WithLabelValues("SetStatus").Observe(elapsed)
	clientCallsForPMProcess.WithLabelValues(
		"SetStatus", c.Encoding, "success").Inc()

	return
}

// ProcessServer is the interface that providers of the service
// Process should implement.
type ProcessServer interface {
	Query(ctx context.Context, req ProcessQueryRequest, pushRep func(ProcessQueryReply)) (err error)
	Control(ctx context.Context, req ProcessControlRequest) (err error)
	Export(ctx context.Context, req ProcessExportRequest) (resp ProcessExportReply, err error)
	Purge(ctx context.Context, req ProcessPurgeRequest, pushRep func(TaskProgress)) (err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForProcess = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Process",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForProcess = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Process",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForProcess = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Process",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForProcess = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Process",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// ProcessHandler provides a NATS subscription handler that can serve a
// subscription using a given ProcessServer implementation.
type ProcessHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server ProcessServer
}

func NewProcessHandler(ctx context.Context, nc nrpc.NatsConn, s ProcessServer) *ProcessHandler {
	return &ProcessHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *ProcessHandler) Subject() string {
	return "xbus.control.process.>"
}

func (h *ProcessHandler) QueryHandler(ctx context.Context, tail []string, msg *nats.Msg) {
	_, encoding, err := nrpc.ParseSubjectTail(0, tail)
	if err != nil {
		log.Printf("Process: Query subject parsing failed:")
	}
	var req ProcessQueryRequest
	if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
		// Handle error
		return
	}

	ctx, cancel := context.WithCancel(ctx)

	keepStreamAlive := nrpc.NewKeepStreamAlive(h.nc, msg.Reply, encoding, cancel)

	var msgCount uint32

	_, nrpcErr := nrpc.CaptureErrors(func() (proto.Message, error) {
		err := h.server.Query(ctx, req, func(rep ProcessQueryReply){
				if err = nrpc.Publish(&rep, nil, h.nc, msg.Reply, encoding); err != nil {
					log.Printf("nrpc: error publishing response")
					cancel()
					return
				}
				msgCount++
			})
		return nil, err
	})
	keepStreamAlive.Stop()

	if nrpcErr != nil {
		nrpc.Publish(nil, nrpcErr, h.nc, msg.Reply, encoding)
	} else {
		nrpc.Publish(
			nil, &nrpc.Error{Type: nrpc.Error_EOS, MsgCount: msgCount},
			h.nc, msg.Reply, encoding)
	}
}

func (h *ProcessHandler) PurgeHandler(ctx context.Context, tail []string, msg *nats.Msg) {
	_, encoding, err := nrpc.ParseSubjectTail(0, tail)
	if err != nil {
		log.Printf("Process: Purge subject parsing failed:")
	}
	var req ProcessPurgeRequest
	if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
		// Handle error
		return
	}

	ctx, cancel := context.WithCancel(ctx)

	keepStreamAlive := nrpc.NewKeepStreamAlive(h.nc, msg.Reply, encoding, cancel)

	var msgCount uint32

	_, nrpcErr := nrpc.CaptureErrors(func() (proto.Message, error) {
		err := h.server.Purge(ctx, req, func(rep TaskProgress){
				if err = nrpc.Publish(&rep, nil, h.nc, msg.Reply, encoding); err != nil {
					log.Printf("nrpc: error publishing response")
					cancel()
					return
				}
				msgCount++
			})
		return nil, err
	})
	keepStreamAlive.Stop()

	if nrpcErr != nil {
		nrpc.Publish(nil, nrpcErr, h.nc, msg.Reply, encoding)
	} else {
		nrpc.Publish(
			nil, &nrpc.Error{Type: nrpc.Error_EOS, MsgCount: msgCount},
			h.nc, msg.Reply, encoding)
	}
}

func (h *ProcessHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus.control", 0, "process", 0, msg.Subject)
	if err != nil {
		log.Printf("ProcessHanlder: Process subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "query":
		h.QueryHandler(ctx, tail, msg)
		return
	case "control":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("ControlHanlder: Control subject parsing failed: %v", err)
			break
		}
		var req ProcessControlRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("ControlHandler: Control request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForProcess.WithLabelValues(
				"Control", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					var innerResp nrpc.Void
					err := h.server.Control(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("ControlHandler: Control handler failed: %s", replyError.Error())
				serverRequestsForProcess.WithLabelValues(
					"Control", encoding, "handler_fail").Inc()
			}
		}
	case "export":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("ExportHanlder: Export subject parsing failed: %v", err)
			break
		}
		var req ProcessExportRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("ExportHandler: Export request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForProcess.WithLabelValues(
				"Export", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Export(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("ExportHandler: Export handler failed: %s", replyError.Error())
				serverRequestsForProcess.WithLabelValues(
					"Export", encoding, "handler_fail").Inc()
			}
		}
	case "purge":
		h.PurgeHandler(ctx, tail, msg)
		return
	default:
		log.Printf("ProcessHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForProcess.WithLabelValues(
			"Process", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForProcess.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForProcess.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForProcess.WithLabelValues(name).Observe(elapsed)
}

type ProcessClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewProcessClient(nc nrpc.NatsConn) *ProcessClient {
	return &ProcessClient{
		nc:      nc,
		PkgSubject: "xbus.control",
		Subject: "process",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *ProcessClient) Query(
	ctx context.Context,
	req ProcessQueryRequest,
	cb func (context.Context, ProcessQueryReply),
) error {
	start := time.Now()
	subject := c.PkgSubject + "." + c.Subject + "." + "query";

	sub, err := nrpc.StreamCall(ctx, c.nc, subject, &req, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForProcess.WithLabelValues(
			"Query", c.Encoding, "error").Inc()
		return err
	}

	var res ProcessQueryReply
	for {
		err = sub.Next(&res)
		if err != nil {
			break
		}
		cb(ctx, res)
	}
	if err == nrpc.ErrEOS {
		err = nil
	}
	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForProcess.WithLabelValues("Query").Observe(elapsed)
	clientCallsForProcess.WithLabelValues(
		"Query", c.Encoding, "success").Inc()
	return err
}

func (c *ProcessClient) Control(req ProcessControlRequest) (err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "control";

	// call
	var resp github_com_nats_rpc_gogo_nrpc.Void
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForProcess.WithLabelValues(
			"Control", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForProcess.WithLabelValues("Control").Observe(elapsed)
	clientCallsForProcess.WithLabelValues(
		"Control", c.Encoding, "success").Inc()

	return
}

func (c *ProcessClient) Export(req ProcessExportRequest) (resp ProcessExportReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "export";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForProcess.WithLabelValues(
			"Export", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForProcess.WithLabelValues("Export").Observe(elapsed)
	clientCallsForProcess.WithLabelValues(
		"Export", c.Encoding, "success").Inc()

	return
}

func (c *ProcessClient) Purge(
	ctx context.Context,
	req ProcessPurgeRequest,
	cb func (context.Context, TaskProgress),
) error {
	start := time.Now()
	subject := c.PkgSubject + "." + c.Subject + "." + "purge";

	sub, err := nrpc.StreamCall(ctx, c.nc, subject, &req, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForProcess.WithLabelValues(
			"Purge", c.Encoding, "error").Inc()
		return err
	}

	var res TaskProgress
	for {
		err = sub.Next(&res)
		if err != nil {
			break
		}
		cb(ctx, res)
	}
	if err == nrpc.ErrEOS {
		err = nil
	}
	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForProcess.WithLabelValues("Purge").Observe(elapsed)
	clientCallsForProcess.WithLabelValues(
		"Purge", c.Encoding, "success").Inc()
	return err
}

type Client struct {
	nc      nrpc.NatsConn
	defaultEncoding string
	defaultTimeout time.Duration
	pkgSubject string
	Account *AccountClient
	Actor *ActorClient
	Envelope *EnvelopeClient
	Graph *GraphClient
	Logs *LogsClient
	PMProcess *PMProcessClient
	Process *ProcessClient
}

func NewClient(nc nrpc.NatsConn) *Client {
	c := Client{
		nc: nc,
		defaultEncoding: "protobuf",
		defaultTimeout: 5*time.Second,
		pkgSubject: "xbus.control",
	};
	c.Account = NewAccountClient(nc)
	c.Actor = NewActorClient(nc)
	c.Envelope = NewEnvelopeClient(nc)
	c.Graph = NewGraphClient(nc)
	c.Logs = NewLogsClient(nc)
	c.PMProcess = NewPMProcessClient(nc)
	c.Process = NewProcessClient(nc)
	return &c
}

func (c *Client) SetEncoding(encoding string) {
	c.defaultEncoding = encoding
	if c.Account != nil {
		c.Account.Encoding = encoding
	}
	if c.Actor != nil {
		c.Actor.Encoding = encoding
	}
	if c.Envelope != nil {
		c.Envelope.Encoding = encoding
	}
	if c.Graph != nil {
		c.Graph.Encoding = encoding
	}
	if c.Logs != nil {
		c.Logs.Encoding = encoding
	}
	if c.PMProcess != nil {
		c.PMProcess.Encoding = encoding
	}
	if c.Process != nil {
		c.Process.Encoding = encoding
	}
}

func (c *Client) SetTimeout(t time.Duration) {
	c.defaultTimeout = t
	if c.Account != nil {
		c.Account.Timeout = t
	}
	if c.Actor != nil {
		c.Actor.Timeout = t
	}
	if c.Envelope != nil {
		c.Envelope.Timeout = t
	}
	if c.Graph != nil {
		c.Graph.Timeout = t
	}
	if c.Logs != nil {
		c.Logs.Timeout = t
	}
	if c.PMProcess != nil {
		c.PMProcess.Timeout = t
	}
	if c.Process != nil {
		c.Process.Timeout = t
	}
}

func init() {
	// register metrics for service Account
	prometheus.MustRegister(clientRCTForAccount)
	prometheus.MustRegister(serverHETForAccount)
	prometheus.MustRegister(clientCallsForAccount)
	prometheus.MustRegister(serverRequestsForAccount)
	// register metrics for service Actor
	prometheus.MustRegister(clientRCTForActor)
	prometheus.MustRegister(serverHETForActor)
	prometheus.MustRegister(clientCallsForActor)
	prometheus.MustRegister(serverRequestsForActor)
	// register metrics for service Envelope
	prometheus.MustRegister(clientRCTForEnvelope)
	prometheus.MustRegister(serverHETForEnvelope)
	prometheus.MustRegister(clientCallsForEnvelope)
	prometheus.MustRegister(serverRequestsForEnvelope)
	// register metrics for service Graph
	prometheus.MustRegister(clientRCTForGraph)
	prometheus.MustRegister(serverHETForGraph)
	prometheus.MustRegister(clientCallsForGraph)
	prometheus.MustRegister(serverRequestsForGraph)
	// register metrics for service Logs
	prometheus.MustRegister(clientRCTForLogs)
	prometheus.MustRegister(serverHETForLogs)
	prometheus.MustRegister(clientCallsForLogs)
	prometheus.MustRegister(serverRequestsForLogs)
	// register metrics for service PMProcess
	prometheus.MustRegister(clientRCTForPMProcess)
	prometheus.MustRegister(serverHETForPMProcess)
	prometheus.MustRegister(clientCallsForPMProcess)
	prometheus.MustRegister(serverRequestsForPMProcess)
	// register metrics for service Process
	prometheus.MustRegister(clientRCTForProcess)
	prometheus.MustRegister(serverHETForProcess)
	prometheus.MustRegister(clientCallsForProcess)
	prometheus.MustRegister(serverRequestsForProcess)
}