// Copyright 2016 Orus.io. All rights reserved.

package api

const (
	// Msgbox is were envelopes are sent
	Msgbox = XbusAPIPrefix + "msgbox"

	// DefaultInput is the name of the default input
	DefaultInput = "default"
	// DefaultOutput is the name of the default output
	DefaultOutput = "default"
)

// EmitterMsgboxSubject returns the msgbox for an emitter
func EmitterMsgboxSubject(id UUID) string {
	return Msgbox + "." + id.String()
}

// ActorOutputSubject returns the subject for an actor output
func ActorOutputSubject(id UUID, output string) string {
	return Msgbox + "." + id.String() + ".output." + output
}

// ActorInputSubject returns the subject for an actor input
func ActorInputSubject(id UUID, output string) string {
	return Msgbox + "." + id.String() + ".input." + output
}

var (
	// EnvelopeMaxSize is an approximate maximum size of an envelope
	// A single Envelope should never exceed this size, or it could be
	// rejected by nats
	EnvelopeMaxSize = 750 * 1024 // 750K encrypted should be smaller than 1M
)

// NewEnvelopeAck creates a normal EnvelopeAck
func NewEnvelopeAck(id UUID, status EnvelopeAck_ReceptionStatus) *EnvelopeAck {
	return &EnvelopeAck{
		ID:     id,
		Status: status,
	}
}

// NewEnvelopeAckError prepare a EnvelopeAck from an error
func NewEnvelopeAckError(id UUID, err error) *EnvelopeAck {
	return &EnvelopeAck{
		ID:     id,
		Status: EnvelopeAck_ERROR,
		Reason: err.Error(),
	}
}

// CloseOutput returns a special envelope that closes an output
// without sending any data on it
func CloseOutput(context EnvelopeContext) Envelope {
	return Envelope{
		Context: context,
		Last:    true,
	}
}

// ForwardEnvelope builds a special envelope that indicates to
// the broker that this envelope is the same that was received
// on an input.
// Such an envelope can be sent only once per output, but can
// be sent before the complete envelope was received.
func ForwardEnvelope(envelope *Envelope) Envelope {
	return Envelope{
		ID:      envelope.ID,
		Context: envelope.Context,
		Last:    true,
	}
}
