// This code was autogenerated from xbus/xbus.proto, do not edit.
package api

import (
	"context"
	"log"
	"time"

	"github.com/golang/protobuf/proto"
	nats "github.com/nats-io/go-nats"
	github_com_nats_rpc_gogo_nrpc "github.com/nats-rpc/gogo-nrpc"
	"github.com/prometheus/client_golang/prometheus"
	"github.com/nats-rpc/gogo-nrpc"
)

// ActorProcessServer is the interface that providers of the service
// ActorProcess should implement.
type ActorProcessServer interface {
	ProcessingEnd(ctx context.Context, req ActorProcessingEndRequest) (err error)
	Log(ctx context.Context, req ActorLogRequest) (err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForActorProcess = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ActorProcess",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForActorProcess = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ActorProcess",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForActorProcess = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "ActorProcess",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForActorProcess = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "ActorProcess",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// ActorProcessHandler provides a NATS subscription handler that can serve a
// subscription using a given ActorProcessServer implementation.
type ActorProcessHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server ActorProcessServer
}

func NewActorProcessHandler(ctx context.Context, nc nrpc.NatsConn, s ActorProcessServer) *ActorProcessHandler {
	return &ActorProcessHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *ActorProcessHandler) Subject() string {
	return "xbus.actor.*.>"
}

func (h *ActorProcessHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, svcParams, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "actor", 1, msg.Subject)
	if err != nil {
		log.Printf("ActorProcessHanlder: ActorProcess subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	ctx = context.WithValue(ctx, "nrpc-svc-actorID", svcParams[0])
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "processingend":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("ProcessingEndHanlder: ProcessingEnd subject parsing failed: %v", err)
			break
		}
		var req ActorProcessingEndRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("ProcessingEndHandler: ProcessingEnd request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForActorProcess.WithLabelValues(
				"ProcessingEnd", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					var innerResp nrpc.Void
					err := h.server.ProcessingEnd(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("ProcessingEndHandler: ProcessingEnd handler failed: %s", replyError.Error())
				serverRequestsForActorProcess.WithLabelValues(
					"ProcessingEnd", encoding, "handler_fail").Inc()
			}
		}
	case "log":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("LogHanlder: Log subject parsing failed: %v", err)
			break
		}
		var req ActorLogRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("LogHandler: Log request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForActorProcess.WithLabelValues(
				"Log", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					var innerResp nrpc.Void
					err := h.server.Log(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("LogHandler: Log handler failed: %s", replyError.Error())
				serverRequestsForActorProcess.WithLabelValues(
					"Log", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("ActorProcessHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForActorProcess.WithLabelValues(
			"ActorProcess", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForActorProcess.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForActorProcess.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForActorProcess.WithLabelValues(name).Observe(elapsed)
}

type ActorProcessClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	SvcParamactorID string
	Encoding string
	Timeout time.Duration
}

func NewActorProcessClient(nc nrpc.NatsConn, svcParamactorID string) *ActorProcessClient {
	return &ActorProcessClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "actor",
		SvcParamactorID: svcParamactorID,
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *ActorProcessClient) ProcessingEnd(req ActorProcessingEndRequest) (err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + c.SvcParamactorID + "." + "processingend";

	// call
	var resp github_com_nats_rpc_gogo_nrpc.Void
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForActorProcess.WithLabelValues(
			"ProcessingEnd", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForActorProcess.WithLabelValues("ProcessingEnd").Observe(elapsed)
	clientCallsForActorProcess.WithLabelValues(
		"ProcessingEnd", c.Encoding, "success").Inc()

	return
}

func (c *ActorProcessClient) Log(req ActorLogRequest) (err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + c.SvcParamactorID + "." + "log";

	// call
	var resp github_com_nats_rpc_gogo_nrpc.Void
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForActorProcess.WithLabelValues(
			"Log", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForActorProcess.WithLabelValues("Log").Observe(elapsed)
	clientCallsForActorProcess.WithLabelValues(
		"Log", c.Encoding, "success").Inc()

	return
}

// ClientAPIServer is the interface that providers of the service
// ClientAPI should implement.
type ClientAPIServer interface {
	GetActors(ctx context.Context) (resp ClientGetActorsReply, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForClientAPI = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ClientAPI",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForClientAPI = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ClientAPI",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForClientAPI = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "ClientAPI",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForClientAPI = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "ClientAPI",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// ClientAPIHandler provides a NATS subscription handler that can serve a
// subscription using a given ClientAPIServer implementation.
type ClientAPIHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server ClientAPIServer
}

func NewClientAPIHandler(ctx context.Context, nc nrpc.NatsConn, s ClientAPIServer) *ClientAPIHandler {
	return &ClientAPIHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *ClientAPIHandler) Subject() string {
	return "xbus.client.*.>"
}

func (h *ClientAPIHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, svcParams, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "client", 1, msg.Subject)
	if err != nil {
		log.Printf("ClientAPIHanlder: ClientAPI subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	ctx = context.WithValue(ctx, "nrpc-svc-accountID", svcParams[0])
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "getactors":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("GetActorsHanlder: GetActors subject parsing failed: %v", err)
			break
		}
		var req github_com_nats_rpc_gogo_nrpc.Void
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("GetActorsHandler: GetActors request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForClientAPI.WithLabelValues(
				"GetActors", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.GetActors(ctx)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("GetActorsHandler: GetActors handler failed: %s", replyError.Error())
				serverRequestsForClientAPI.WithLabelValues(
					"GetActors", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("ClientAPIHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForClientAPI.WithLabelValues(
			"ClientAPI", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForClientAPI.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForClientAPI.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForClientAPI.WithLabelValues(name).Observe(elapsed)
}

type ClientAPIClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	SvcParamaccountID string
	Encoding string
	Timeout time.Duration
}

func NewClientAPIClient(nc nrpc.NatsConn, svcParamaccountID string) *ClientAPIClient {
	return &ClientAPIClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "client",
		SvcParamaccountID: svcParamaccountID,
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *ClientAPIClient) GetActors() (resp ClientGetActorsReply, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + c.SvcParamaccountID + "." + "getactors";

	// call
	var req github_com_nats_rpc_gogo_nrpc.Void
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForClientAPI.WithLabelValues(
			"GetActors", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForClientAPI.WithLabelValues("GetActors").Observe(elapsed)
	clientCallsForClientAPI.WithLabelValues(
		"GetActors", c.Encoding, "success").Inc()

	return
}

// WhoAmIServer is the interface that providers of the service
// WhoAmI should implement.
type WhoAmIServer interface {
	WhoAmI(ctx context.Context, apiKey string) (resp Account, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForWhoAmI = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "WhoAmI",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForWhoAmI = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "WhoAmI",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForWhoAmI = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "WhoAmI",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForWhoAmI = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "WhoAmI",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// WhoAmIHandler provides a NATS subscription handler that can serve a
// subscription using a given WhoAmIServer implementation.
type WhoAmIHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server WhoAmIServer
}

func NewWhoAmIHandler(ctx context.Context, nc nrpc.NatsConn, s WhoAmIServer) *WhoAmIHandler {
	return &WhoAmIHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *WhoAmIHandler) Subject() string {
	return "xbus.client.>"
}

func (h *WhoAmIHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "client", 0, msg.Subject)
	if err != nil {
		log.Printf("WhoAmIHanlder: WhoAmI subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "whoami":
		var mtParams []string
		mtParams, encoding, err = nrpc.ParseSubjectTail(1, tail)
		if err != nil {
			log.Printf("WhoAmIHanlder: WhoAmI subject parsing failed: %v", err)
			break
		}
		var req github_com_nats_rpc_gogo_nrpc.Void
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("WhoAmIHandler: WhoAmI request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForWhoAmI.WithLabelValues(
				"WhoAmI", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.WhoAmI(ctx, mtParams[0])
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("WhoAmIHandler: WhoAmI handler failed: %s", replyError.Error())
				serverRequestsForWhoAmI.WithLabelValues(
					"WhoAmI", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("WhoAmIHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForWhoAmI.WithLabelValues(
			"WhoAmI", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForWhoAmI.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForWhoAmI.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForWhoAmI.WithLabelValues(name).Observe(elapsed)
}

type WhoAmIClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewWhoAmIClient(nc nrpc.NatsConn) *WhoAmIClient {
	return &WhoAmIClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "client",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *WhoAmIClient) WhoAmI(apiKey string, ) (resp Account, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "whoami" + "." + apiKey;

	// call
	var req github_com_nats_rpc_gogo_nrpc.Void
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForWhoAmI.WithLabelValues(
			"WhoAmI", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForWhoAmI.WithLabelValues("WhoAmI").Observe(elapsed)
	clientCallsForWhoAmI.WithLabelValues(
		"WhoAmI", c.Encoding, "success").Inc()

	return
}

// ProcessStateServer is the interface that providers of the service
// ProcessState should implement.
type ProcessStateServer interface {
}

var (
	// The request completion time, measured at client-side.
	clientRCTForProcessState = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ProcessState",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForProcessState = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ProcessState",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForProcessState = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "ProcessState",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForProcessState = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "ProcessState",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// ProcessStateHandler provides a NATS subscription handler that can serve a
// subscription using a given ProcessStateServer implementation.
type ProcessStateHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server ProcessStateServer
}

func NewProcessStateHandler(ctx context.Context, nc nrpc.NatsConn, s ProcessStateServer) *ProcessStateHandler {
	return &ProcessStateHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *ProcessStateHandler) Subject() string {
	return "xbus.processstate.*.>"
}

func (h *ProcessStateHandler) EnvelopePublish(svcemitterID string, mtenvelopeID string, msg EmitterEnvelopeState) error {
	rawMsg, err := nrpc.Marshal("protobuf", &msg)
	if err != nil {
		log.Printf("ProcessStateHandler.EnvelopePublish: error marshaling the message: %s", err)
		return err
	}
	subject := "xbus."+ "processstate." + svcemitterID + "."+ "envelope" + "." + mtenvelopeID
	return h.nc.Publish(subject, rawMsg)
}

type ProcessStateClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	SvcParamemitterID string
	Encoding string
	Timeout time.Duration
}

func NewProcessStateClient(nc nrpc.NatsConn, svcParamemitterID string) *ProcessStateClient {
	return &ProcessStateClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "processstate",
		SvcParamemitterID: svcParamemitterID,
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *ProcessStateClient) EnvelopeSubject(
	mtenvelopeID string,
) string {
	return c.PkgSubject + "." + c.Subject + "." + c.SvcParamemitterID + "." + "envelope" + "." + mtenvelopeID
}

type ProcessStateEnvelopeSubscription struct {
	*nats.Subscription
}

func (s *ProcessStateEnvelopeSubscription) Next(timeout time.Duration) (next EmitterEnvelopeState, err error) {
	msg, err := s.Subscription.NextMsg(timeout)
	if err != nil {
		return
	}
	err = nrpc.Unmarshal("protobuf", msg.Data, &next)
	return
}

func (c *ProcessStateClient) EnvelopeSubscribeSync(
	mtenvelopeID string,
) (sub *ProcessStateEnvelopeSubscription, err error) {
	subject := c.EnvelopeSubject(
		mtenvelopeID,
	)
	natsSub, err := c.nc.SubscribeSync(subject)
	if err != nil {
		return
	}
	sub = &ProcessStateEnvelopeSubscription{natsSub}
	return
}

func (c *ProcessStateClient) EnvelopeSubscribe(
	mtenvelopeID string,
	handler func (EmitterEnvelopeState),
) (sub *nats.Subscription, err error) {
	subject := c.EnvelopeSubject(
		mtenvelopeID,
	)
	sub, err = c.nc.Subscribe(subject, func(msg *nats.Msg){
		var pmsg EmitterEnvelopeState
		err := nrpc.Unmarshal("protobuf", msg.Data, &pmsg)
		if err != nil {
			log.Printf("ProcessStateClient.EnvelopeSubscribe: Error decoding, %s", err)
			return
		}
		handler(pmsg)
	})
	return
}

func (c *ProcessStateClient) EnvelopeSubscribeChan(
	mtenvelopeID string,
) (<-chan EmitterEnvelopeState, *nats.Subscription, error) {
	ch := make(chan EmitterEnvelopeState)
	sub, err := c.EnvelopeSubscribe(mtenvelopeID, func (msg EmitterEnvelopeState) {
		ch <- msg
	})
	return ch, sub, err
}

// ActorInputServer is the interface that providers of the service
// ActorInput should implement.
type ActorInputServer interface {
	Input(ctx context.Context, input string, envelopeID string, req Envelope) (resp EnvelopeAck, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForActorInput = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ActorInput",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForActorInput = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "ActorInput",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForActorInput = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "ActorInput",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForActorInput = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "ActorInput",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// ActorInputHandler provides a NATS subscription handler that can serve a
// subscription using a given ActorInputServer implementation.
type ActorInputHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server ActorInputServer
}

func NewActorInputHandler(ctx context.Context, nc nrpc.NatsConn, s ActorInputServer) *ActorInputHandler {
	return &ActorInputHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *ActorInputHandler) Subject() string {
	return "xbus.msgbox.*.>"
}

func (h *ActorInputHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, svcParams, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "msgbox", 1, msg.Subject)
	if err != nil {
		log.Printf("ActorInputHanlder: ActorInput subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	ctx = context.WithValue(ctx, "nrpc-svc-actorID", svcParams[0])
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "input":
		var mtParams []string
		mtParams, encoding, err = nrpc.ParseSubjectTail(2, tail)
		if err != nil {
			log.Printf("InputHanlder: Input subject parsing failed: %v", err)
			break
		}
		var req Envelope
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("InputHandler: Input request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForActorInput.WithLabelValues(
				"Input", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Input(ctx, mtParams[0], mtParams[1], req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("InputHandler: Input handler failed: %s", replyError.Error())
				serverRequestsForActorInput.WithLabelValues(
					"Input", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("ActorInputHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForActorInput.WithLabelValues(
			"ActorInput", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForActorInput.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForActorInput.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForActorInput.WithLabelValues(name).Observe(elapsed)
}

type ActorInputClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	SvcParamactorID string
	Encoding string
	Timeout time.Duration
}

func NewActorInputClient(nc nrpc.NatsConn, svcParamactorID string) *ActorInputClient {
	return &ActorInputClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "msgbox",
		SvcParamactorID: svcParamactorID,
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *ActorInputClient) Input(input string, envelopeID string, req Envelope) (resp EnvelopeAck, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + c.SvcParamactorID + "." + "input" + "." + input + "." + envelopeID;

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForActorInput.WithLabelValues(
			"Input", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForActorInput.WithLabelValues("Input").Observe(elapsed)
	clientCallsForActorInput.WithLabelValues(
		"Input", c.Encoding, "success").Inc()

	return
}

// BrokerServer is the interface that providers of the service
// Broker should implement.
type BrokerServer interface {
	Output(ctx context.Context, output string, req Envelope) (resp EnvelopeAck, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForBroker = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Broker",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForBroker = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Broker",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForBroker = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Broker",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForBroker = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Broker",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// BrokerHandler provides a NATS subscription handler that can serve a
// subscription using a given BrokerServer implementation.
type BrokerHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server BrokerServer
}

func NewBrokerHandler(ctx context.Context, nc nrpc.NatsConn, s BrokerServer) *BrokerHandler {
	return &BrokerHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *BrokerHandler) Subject() string {
	return "xbus.msgbox.*.>"
}

func (h *BrokerHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, svcParams, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "msgbox", 1, msg.Subject)
	if err != nil {
		log.Printf("BrokerHanlder: Broker subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	ctx = context.WithValue(ctx, "nrpc-svc-actorID", svcParams[0])
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "output":
		var mtParams []string
		mtParams, encoding, err = nrpc.ParseSubjectTail(1, tail)
		if err != nil {
			log.Printf("OutputHanlder: Output subject parsing failed: %v", err)
			break
		}
		var req Envelope
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("OutputHandler: Output request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForBroker.WithLabelValues(
				"Output", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Output(ctx, mtParams[0], req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("OutputHandler: Output handler failed: %s", replyError.Error())
				serverRequestsForBroker.WithLabelValues(
					"Output", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("BrokerHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForBroker.WithLabelValues(
			"Broker", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForBroker.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForBroker.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForBroker.WithLabelValues(name).Observe(elapsed)
}

type BrokerClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	SvcParamactorID string
	Encoding string
	Timeout time.Duration
}

func NewBrokerClient(nc nrpc.NatsConn, svcParamactorID string) *BrokerClient {
	return &BrokerClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "msgbox",
		SvcParamactorID: svcParamactorID,
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *BrokerClient) Output(output string, req Envelope) (resp EnvelopeAck, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + c.SvcParamactorID + "." + "output" + "." + output;

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForBroker.WithLabelValues(
			"Output", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForBroker.WithLabelValues("Output").Observe(elapsed)
	clientCallsForBroker.WithLabelValues(
		"Output", c.Encoding, "success").Inc()

	return
}

// CoreEventsServer is the interface that providers of the service
// CoreEvents should implement.
type CoreEventsServer interface {
}

var (
	// The request completion time, measured at client-side.
	clientRCTForCoreEvents = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "CoreEvents",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForCoreEvents = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "CoreEvents",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForCoreEvents = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "CoreEvents",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForCoreEvents = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "CoreEvents",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// CoreEventsHandler provides a NATS subscription handler that can serve a
// subscription using a given CoreEventsServer implementation.
type CoreEventsHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server CoreEventsServer
}

func NewCoreEventsHandler(ctx context.Context, nc nrpc.NatsConn, s CoreEventsServer) *CoreEventsHandler {
	return &CoreEventsHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *CoreEventsHandler) Subject() string {
	return "xbus.core.events.>"
}

func (h *CoreEventsHandler) ActorPublish(msg CoreEvent) error {
	rawMsg, err := nrpc.Marshal("protobuf", &msg)
	if err != nil {
		log.Printf("CoreEventsHandler.ActorPublish: error marshaling the message: %s", err)
		return err
	}
	subject := "xbus."+ "core.events."+ "actor"
	return h.nc.Publish(subject, rawMsg)
}

func (h *CoreEventsHandler) ProcessPublish(msg CoreEvent) error {
	rawMsg, err := nrpc.Marshal("protobuf", &msg)
	if err != nil {
		log.Printf("CoreEventsHandler.ProcessPublish: error marshaling the message: %s", err)
		return err
	}
	subject := "xbus."+ "core.events."+ "process"
	return h.nc.Publish(subject, rawMsg)
}

type CoreEventsClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewCoreEventsClient(nc nrpc.NatsConn) *CoreEventsClient {
	return &CoreEventsClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "core.events",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *CoreEventsClient) ActorSubject(
	
) string {
	return c.PkgSubject + "." + c.Subject + "." + "actor"
}

type CoreEventsActorSubscription struct {
	*nats.Subscription
}

func (s *CoreEventsActorSubscription) Next(timeout time.Duration) (next CoreEvent, err error) {
	msg, err := s.Subscription.NextMsg(timeout)
	if err != nil {
		return
	}
	err = nrpc.Unmarshal("protobuf", msg.Data, &next)
	return
}

func (c *CoreEventsClient) ActorSubscribeSync(
	
) (sub *CoreEventsActorSubscription, err error) {
	subject := c.ActorSubject(
		
	)
	natsSub, err := c.nc.SubscribeSync(subject)
	if err != nil {
		return
	}
	sub = &CoreEventsActorSubscription{natsSub}
	return
}

func (c *CoreEventsClient) ActorSubscribe(
	
	handler func (CoreEvent),
) (sub *nats.Subscription, err error) {
	subject := c.ActorSubject(
		
	)
	sub, err = c.nc.Subscribe(subject, func(msg *nats.Msg){
		var pmsg CoreEvent
		err := nrpc.Unmarshal("protobuf", msg.Data, &pmsg)
		if err != nil {
			log.Printf("CoreEventsClient.ActorSubscribe: Error decoding, %s", err)
			return
		}
		handler(pmsg)
	})
	return
}

func (c *CoreEventsClient) ActorSubscribeChan(
	
) (<-chan CoreEvent, *nats.Subscription, error) {
	ch := make(chan CoreEvent)
	sub, err := c.ActorSubscribe(func (msg CoreEvent) {
		ch <- msg
	})
	return ch, sub, err
}

func (c *CoreEventsClient) ProcessSubject(
	
) string {
	return c.PkgSubject + "." + c.Subject + "." + "process"
}

type CoreEventsProcessSubscription struct {
	*nats.Subscription
}

func (s *CoreEventsProcessSubscription) Next(timeout time.Duration) (next CoreEvent, err error) {
	msg, err := s.Subscription.NextMsg(timeout)
	if err != nil {
		return
	}
	err = nrpc.Unmarshal("protobuf", msg.Data, &next)
	return
}

func (c *CoreEventsClient) ProcessSubscribeSync(
	
) (sub *CoreEventsProcessSubscription, err error) {
	subject := c.ProcessSubject(
		
	)
	natsSub, err := c.nc.SubscribeSync(subject)
	if err != nil {
		return
	}
	sub = &CoreEventsProcessSubscription{natsSub}
	return
}

func (c *CoreEventsClient) ProcessSubscribe(
	
	handler func (CoreEvent),
) (sub *nats.Subscription, err error) {
	subject := c.ProcessSubject(
		
	)
	sub, err = c.nc.Subscribe(subject, func(msg *nats.Msg){
		var pmsg CoreEvent
		err := nrpc.Unmarshal("protobuf", msg.Data, &pmsg)
		if err != nil {
			log.Printf("CoreEventsClient.ProcessSubscribe: Error decoding, %s", err)
			return
		}
		handler(pmsg)
	})
	return
}

func (c *CoreEventsClient) ProcessSubscribeChan(
	
) (<-chan CoreEvent, *nats.Subscription, error) {
	ch := make(chan CoreEvent)
	sub, err := c.ProcessSubscribe(func (msg CoreEvent) {
		ch <- msg
	})
	return ch, sub, err
}

// EmissionServer is the interface that providers of the service
// Emission should implement.
type EmissionServer interface {
	Signal(ctx context.Context, req EmissionSignal)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForEmission = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Emission",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForEmission = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Emission",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForEmission = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Emission",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForEmission = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Emission",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// EmissionHandler provides a NATS subscription handler that can serve a
// subscription using a given EmissionServer implementation.
type EmissionHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server EmissionServer
}

func NewEmissionHandler(ctx context.Context, nc nrpc.NatsConn, s EmissionServer) *EmissionHandler {
	return &EmissionHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *EmissionHandler) Subject() string {
	return "xbus.services.emission.>"
}

func (h *EmissionHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "services.emission", 0, msg.Subject)
	if err != nil {
		log.Printf("EmissionHanlder: Emission subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "signal":
		noreply = true
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("SignalHanlder: Signal subject parsing failed: %v", err)
			break
		}
		var req EmissionSignal
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("SignalHandler: Signal request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForEmission.WithLabelValues(
				"Signal", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){var innerResp nrpc.NoReply
					h.server.Signal(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("SignalHandler: Signal handler failed: %s", replyError.Error())
				serverRequestsForEmission.WithLabelValues(
					"Signal", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("EmissionHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForEmission.WithLabelValues(
			"Emission", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForEmission.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForEmission.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForEmission.WithLabelValues(name).Observe(elapsed)
}

type EmissionClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewEmissionClient(nc nrpc.NatsConn) *EmissionClient {
	return &EmissionClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "services.emission",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *EmissionClient) Signal(req EmissionSignal) (err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "signal";

	// call
	var resp github_com_nats_rpc_gogo_nrpc.NoReply
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForEmission.WithLabelValues(
			"Signal", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForEmission.WithLabelValues("Signal").Observe(elapsed)
	clientCallsForEmission.WithLabelValues(
		"Signal", c.Encoding, "success").Inc()

	return
}

// SvcRegistrationServer is the interface that providers of the service
// SvcRegistration should implement.
type SvcRegistrationServer interface {
	Register(ctx context.Context, req RegistrationRequest) (resp Registration, err error)
	Update(ctx context.Context, accountID string, req RegistrationRequest) (resp Registration, err error)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForSvcRegistration = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "SvcRegistration",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForSvcRegistration = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "SvcRegistration",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForSvcRegistration = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "SvcRegistration",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForSvcRegistration = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "SvcRegistration",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// SvcRegistrationHandler provides a NATS subscription handler that can serve a
// subscription using a given SvcRegistrationServer implementation.
type SvcRegistrationHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server SvcRegistrationServer
}

func NewSvcRegistrationHandler(ctx context.Context, nc nrpc.NatsConn, s SvcRegistrationServer) *SvcRegistrationHandler {
	return &SvcRegistrationHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *SvcRegistrationHandler) Subject() string {
	return "xbus.registration.>"
}

func (h *SvcRegistrationHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "registration", 0, msg.Subject)
	if err != nil {
		log.Printf("SvcRegistrationHanlder: SvcRegistration subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "register":
		_, encoding, err = nrpc.ParseSubjectTail(0, tail)
		if err != nil {
			log.Printf("RegisterHanlder: Register subject parsing failed: %v", err)
			break
		}
		var req RegistrationRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("RegisterHandler: Register request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForSvcRegistration.WithLabelValues(
				"Register", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Register(ctx, req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("RegisterHandler: Register handler failed: %s", replyError.Error())
				serverRequestsForSvcRegistration.WithLabelValues(
					"Register", encoding, "handler_fail").Inc()
			}
		}
	case "update":
		var mtParams []string
		mtParams, encoding, err = nrpc.ParseSubjectTail(1, tail)
		if err != nil {
			log.Printf("UpdateHanlder: Update subject parsing failed: %v", err)
			break
		}
		var req RegistrationRequest
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("UpdateHandler: Update request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForSvcRegistration.WithLabelValues(
				"Update", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){
					innerResp, err := h.server.Update(ctx, mtParams[0], req)
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("UpdateHandler: Update handler failed: %s", replyError.Error())
				serverRequestsForSvcRegistration.WithLabelValues(
					"Update", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("SvcRegistrationHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForSvcRegistration.WithLabelValues(
			"SvcRegistration", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForSvcRegistration.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForSvcRegistration.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForSvcRegistration.WithLabelValues(name).Observe(elapsed)
}

type SvcRegistrationClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewSvcRegistrationClient(nc nrpc.NatsConn) *SvcRegistrationClient {
	return &SvcRegistrationClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "registration",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *SvcRegistrationClient) Register(req RegistrationRequest) (resp Registration, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "register";

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForSvcRegistration.WithLabelValues(
			"Register", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForSvcRegistration.WithLabelValues("Register").Observe(elapsed)
	clientCallsForSvcRegistration.WithLabelValues(
		"Register", c.Encoding, "success").Inc()

	return
}

func (c *SvcRegistrationClient) Update(accountID string, req RegistrationRequest) (resp Registration, err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "update" + "." + accountID;

	// call
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForSvcRegistration.WithLabelValues(
			"Update", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForSvcRegistration.WithLabelValues("Update").Observe(elapsed)
	clientCallsForSvcRegistration.WithLabelValues(
		"Update", c.Encoding, "success").Inc()

	return
}

// SentinelServer is the interface that providers of the service
// Sentinel should implement.
type SentinelServer interface {
	ActorReady(ctx context.Context, actorID string)
}

var (
	// The request completion time, measured at client-side.
	clientRCTForSentinel = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_client_request_completion_time_seconds",
			Help:       "The request completion time for calls, measured client-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Sentinel",
			},
		},
		[]string{"method"})

	// The handler execution time, measured at server-side.
	serverHETForSentinel = prometheus.NewSummaryVec(
		prometheus.SummaryOpts{
			Name:       "nrpc_server_handler_execution_time_seconds",
			Help:       "The handler execution time for calls, measured server-side.",
			Objectives: map[float64]float64{0.9: 0.01, 0.95: 0.01, 0.99: 0.001},
			ConstLabels: map[string]string{
				"service": "Sentinel",
			},
		},
		[]string{"method"})

	// The counts of calls made by the client, classified by result type.
	clientCallsForSentinel = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_client_calls_count",
			Help: "The count of calls made by the client.",
			ConstLabels: map[string]string{
				"service": "Sentinel",
			},
		},
		[]string{"method", "encoding", "result_type"})

	// The counts of requests handled by the server, classified by result type.
	serverRequestsForSentinel = prometheus.NewCounterVec(
		prometheus.CounterOpts{
			Name: "nrpc_server_requests_count",
			Help: "The count of requests handled by the server.",
			ConstLabels: map[string]string{
				"service": "Sentinel",
			},
		},
		[]string{"method", "encoding", "result_type"})
)

// SentinelHandler provides a NATS subscription handler that can serve a
// subscription using a given SentinelServer implementation.
type SentinelHandler struct {
	ctx    context.Context
	nc     nrpc.NatsConn
	server SentinelServer
}

func NewSentinelHandler(ctx context.Context, nc nrpc.NatsConn, s SentinelServer) *SentinelHandler {
	return &SentinelHandler{
		ctx:    ctx,
		nc:     nc,
		server: s,
	}
}

func (h *SentinelHandler) Subject() string {
	return "xbus.sentinel.>"
}

func (h *SentinelHandler) Handler(msg *nats.Msg) {
	var encoding string
	var noreply bool
	// extract method name & encoding from subject
	_, _, name, tail, err := nrpc.ParseSubject(
		"xbus", 0, "sentinel", 0, msg.Subject)
	if err != nil {
		log.Printf("SentinelHanlder: Sentinel subject parsing failed: %v", err)
		return
	}

	ctx := h.ctx
	// call handler and form response
	var resp proto.Message
	var replyError *nrpc.Error
	var elapsed float64
	switch name {
	case "actorready":
		var mtParams []string
		noreply = true
		mtParams, encoding, err = nrpc.ParseSubjectTail(1, tail)
		if err != nil {
			log.Printf("ActorReadyHanlder: ActorReady subject parsing failed: %v", err)
			break
		}
		var req github_com_nats_rpc_gogo_nrpc.Void
		if err := nrpc.Unmarshal(encoding, msg.Data, &req); err != nil {
			log.Printf("ActorReadyHandler: ActorReady request unmarshal failed: %v", err)
			replyError = &nrpc.Error{
				Type: nrpc.Error_CLIENT,
				Message: "bad request received: " + err.Error(),
			}
			serverRequestsForSentinel.WithLabelValues(
				"ActorReady", encoding, "unmarshal_fail").Inc()
		} else {
			start := time.Now()
			resp, replyError = nrpc.CaptureErrors(
				func()(proto.Message, error){var innerResp nrpc.NoReply
					h.server.ActorReady(ctx, mtParams[0])
					if err != nil {
						return nil, err
					}
					return &innerResp, err
				})
			elapsed = time.Since(start).Seconds()
			if replyError != nil {
				log.Printf("ActorReadyHandler: ActorReady handler failed: %s", replyError.Error())
				serverRequestsForSentinel.WithLabelValues(
					"ActorReady", encoding, "handler_fail").Inc()
			}
		}
	default:
		log.Printf("SentinelHandler: unknown name %q", name)
		replyError = &nrpc.Error{
			Type: nrpc.Error_CLIENT,
			Message: "unknown name: " + name,
		}
		serverRequestsForSentinel.WithLabelValues(
			"Sentinel", encoding, "name_fail").Inc()
	}


	if !noreply {
		// encode and send response
		err = nrpc.Publish(resp, replyError, h.nc, msg.Reply, encoding) // error is logged
	} else {
		err = nil
	}
	if err != nil {
		serverRequestsForSentinel.WithLabelValues(
			name, encoding, "sendreply_fail").Inc()
	} else if replyError == nil {
		serverRequestsForSentinel.WithLabelValues(
			name, encoding, "success").Inc()
	}

	// report metric to Prometheus
	serverHETForSentinel.WithLabelValues(name).Observe(elapsed)
}

type SentinelClient struct {
	nc      nrpc.NatsConn
	PkgSubject string
	Subject string
	Encoding string
	Timeout time.Duration
}

func NewSentinelClient(nc nrpc.NatsConn) *SentinelClient {
	return &SentinelClient{
		nc:      nc,
		PkgSubject: "xbus",
		Subject: "sentinel",
		Encoding: "protobuf",
		Timeout: 5 * time.Second,
	}
}

func (c *SentinelClient) ActorReady(actorID string, ) (err error) {
	start := time.Now()

	subject := c.PkgSubject + "." + c.Subject + "." + "actorready" + "." + actorID;

	// call
	var req github_com_nats_rpc_gogo_nrpc.Void
	var resp github_com_nats_rpc_gogo_nrpc.NoReply
	err = nrpc.Call(&req, &resp, c.nc, subject, c.Encoding, c.Timeout)
	if err != nil {
		clientCallsForSentinel.WithLabelValues(
			"ActorReady", c.Encoding, "call_fail").Inc()
		return // already logged
	}

	// report total time taken to Prometheus
	elapsed := time.Since(start).Seconds()
	clientRCTForSentinel.WithLabelValues("ActorReady").Observe(elapsed)
	clientCallsForSentinel.WithLabelValues(
		"ActorReady", c.Encoding, "success").Inc()

	return
}

type Client struct {
	nc      nrpc.NatsConn
	defaultEncoding string
	defaultTimeout time.Duration
	pkgSubject string
	ActorProcess *ActorProcessClient
	ClientAPI *ClientAPIClient
	WhoAmI *WhoAmIClient
	ProcessState *ProcessStateClient
	ActorInput *ActorInputClient
	Broker *BrokerClient
	CoreEvents *CoreEventsClient
	Emission *EmissionClient
	SvcRegistration *SvcRegistrationClient
	Sentinel *SentinelClient
}

func NewClient(nc nrpc.NatsConn) *Client {
	c := Client{
		nc: nc,
		defaultEncoding: "protobuf",
		defaultTimeout: 5*time.Second,
		pkgSubject: "xbus",
	};
	c.WhoAmI = NewWhoAmIClient(nc)
	c.CoreEvents = NewCoreEventsClient(nc)
	c.Emission = NewEmissionClient(nc)
	c.SvcRegistration = NewSvcRegistrationClient(nc)
	c.Sentinel = NewSentinelClient(nc)
	return &c
}

func (c *Client) SetEncoding(encoding string) {
	c.defaultEncoding = encoding
	if c.ActorProcess != nil {
		c.ActorProcess.Encoding = encoding
	}
	if c.ClientAPI != nil {
		c.ClientAPI.Encoding = encoding
	}
	if c.WhoAmI != nil {
		c.WhoAmI.Encoding = encoding
	}
	if c.ProcessState != nil {
		c.ProcessState.Encoding = encoding
	}
	if c.ActorInput != nil {
		c.ActorInput.Encoding = encoding
	}
	if c.Broker != nil {
		c.Broker.Encoding = encoding
	}
	if c.CoreEvents != nil {
		c.CoreEvents.Encoding = encoding
	}
	if c.Emission != nil {
		c.Emission.Encoding = encoding
	}
	if c.SvcRegistration != nil {
		c.SvcRegistration.Encoding = encoding
	}
	if c.Sentinel != nil {
		c.Sentinel.Encoding = encoding
	}
}

func (c *Client) SetTimeout(t time.Duration) {
	c.defaultTimeout = t
	if c.ActorProcess != nil {
		c.ActorProcess.Timeout = t
	}
	if c.ClientAPI != nil {
		c.ClientAPI.Timeout = t
	}
	if c.WhoAmI != nil {
		c.WhoAmI.Timeout = t
	}
	if c.ProcessState != nil {
		c.ProcessState.Timeout = t
	}
	if c.ActorInput != nil {
		c.ActorInput.Timeout = t
	}
	if c.Broker != nil {
		c.Broker.Timeout = t
	}
	if c.CoreEvents != nil {
		c.CoreEvents.Timeout = t
	}
	if c.Emission != nil {
		c.Emission.Timeout = t
	}
	if c.SvcRegistration != nil {
		c.SvcRegistration.Timeout = t
	}
	if c.Sentinel != nil {
		c.Sentinel.Timeout = t
	}
}

func (c *Client) SetActorProcessParams(
	actorID string,
) {
	c.ActorProcess = NewActorProcessClient(
		c.nc,
		actorID,
	)
	c.ActorProcess.Encoding = c.defaultEncoding
	c.ActorProcess.Timeout = c.defaultTimeout
}

func (c *Client) NewActorProcess(
	actorID string,
) *ActorProcessClient {
	client := NewActorProcessClient(
		c.nc,
		actorID,
	)
	client.Encoding = c.defaultEncoding
	client.Timeout = c.defaultTimeout
	return client
}

func (c *Client) SetClientAPIParams(
	accountID string,
) {
	c.ClientAPI = NewClientAPIClient(
		c.nc,
		accountID,
	)
	c.ClientAPI.Encoding = c.defaultEncoding
	c.ClientAPI.Timeout = c.defaultTimeout
}

func (c *Client) NewClientAPI(
	accountID string,
) *ClientAPIClient {
	client := NewClientAPIClient(
		c.nc,
		accountID,
	)
	client.Encoding = c.defaultEncoding
	client.Timeout = c.defaultTimeout
	return client
}

func (c *Client) SetProcessStateParams(
	emitterID string,
) {
	c.ProcessState = NewProcessStateClient(
		c.nc,
		emitterID,
	)
	c.ProcessState.Encoding = c.defaultEncoding
	c.ProcessState.Timeout = c.defaultTimeout
}

func (c *Client) NewProcessState(
	emitterID string,
) *ProcessStateClient {
	client := NewProcessStateClient(
		c.nc,
		emitterID,
	)
	client.Encoding = c.defaultEncoding
	client.Timeout = c.defaultTimeout
	return client
}

func (c *Client) SetActorInputParams(
	actorID string,
) {
	c.ActorInput = NewActorInputClient(
		c.nc,
		actorID,
	)
	c.ActorInput.Encoding = c.defaultEncoding
	c.ActorInput.Timeout = c.defaultTimeout
}

func (c *Client) NewActorInput(
	actorID string,
) *ActorInputClient {
	client := NewActorInputClient(
		c.nc,
		actorID,
	)
	client.Encoding = c.defaultEncoding
	client.Timeout = c.defaultTimeout
	return client
}

func (c *Client) SetBrokerParams(
	actorID string,
) {
	c.Broker = NewBrokerClient(
		c.nc,
		actorID,
	)
	c.Broker.Encoding = c.defaultEncoding
	c.Broker.Timeout = c.defaultTimeout
}

func (c *Client) NewBroker(
	actorID string,
) *BrokerClient {
	client := NewBrokerClient(
		c.nc,
		actorID,
	)
	client.Encoding = c.defaultEncoding
	client.Timeout = c.defaultTimeout
	return client
}

func init() {
	// register metrics for service ActorProcess
	prometheus.MustRegister(clientRCTForActorProcess)
	prometheus.MustRegister(serverHETForActorProcess)
	prometheus.MustRegister(clientCallsForActorProcess)
	prometheus.MustRegister(serverRequestsForActorProcess)
	// register metrics for service ClientAPI
	prometheus.MustRegister(clientRCTForClientAPI)
	prometheus.MustRegister(serverHETForClientAPI)
	prometheus.MustRegister(clientCallsForClientAPI)
	prometheus.MustRegister(serverRequestsForClientAPI)
	// register metrics for service WhoAmI
	prometheus.MustRegister(clientRCTForWhoAmI)
	prometheus.MustRegister(serverHETForWhoAmI)
	prometheus.MustRegister(clientCallsForWhoAmI)
	prometheus.MustRegister(serverRequestsForWhoAmI)
	// register metrics for service ProcessState
	prometheus.MustRegister(clientRCTForProcessState)
	prometheus.MustRegister(serverHETForProcessState)
	prometheus.MustRegister(clientCallsForProcessState)
	prometheus.MustRegister(serverRequestsForProcessState)
	// register metrics for service ActorInput
	prometheus.MustRegister(clientRCTForActorInput)
	prometheus.MustRegister(serverHETForActorInput)
	prometheus.MustRegister(clientCallsForActorInput)
	prometheus.MustRegister(serverRequestsForActorInput)
	// register metrics for service Broker
	prometheus.MustRegister(clientRCTForBroker)
	prometheus.MustRegister(serverHETForBroker)
	prometheus.MustRegister(clientCallsForBroker)
	prometheus.MustRegister(serverRequestsForBroker)
	// register metrics for service CoreEvents
	prometheus.MustRegister(clientRCTForCoreEvents)
	prometheus.MustRegister(serverHETForCoreEvents)
	prometheus.MustRegister(clientCallsForCoreEvents)
	prometheus.MustRegister(serverRequestsForCoreEvents)
	// register metrics for service Emission
	prometheus.MustRegister(clientRCTForEmission)
	prometheus.MustRegister(serverHETForEmission)
	prometheus.MustRegister(clientCallsForEmission)
	prometheus.MustRegister(serverRequestsForEmission)
	// register metrics for service SvcRegistration
	prometheus.MustRegister(clientRCTForSvcRegistration)
	prometheus.MustRegister(serverHETForSvcRegistration)
	prometheus.MustRegister(clientCallsForSvcRegistration)
	prometheus.MustRegister(serverRequestsForSvcRegistration)
	// register metrics for service Sentinel
	prometheus.MustRegister(clientRCTForSentinel)
	prometheus.MustRegister(serverHETForSentinel)
	prometheus.MustRegister(clientCallsForSentinel)
	prometheus.MustRegister(serverRequestsForSentinel)
}